#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "vector.h"
#include "font.h"
#include "Obj.h"
#include "Saucer.h"
#include "Log.h"

struct _User {
  char *uid;
  char *name;
  Saucer saucer;
};

struct _Genre {
  char *name;
  int image_width;
  int image_height;
  unsigned char *image;
  ObjList questions;
};

struct _Question {
  char *qid;
  int score;
  int image_width;
  int image_height;
  unsigned char *image;
  Genre genre;
};

struct _Reply {
  User user;
  Question question;
  int t;
};

struct _Clear {
  User user;
  Question question;
  int t;
};

User User_Create(char *uid, char *name, Saucer saucer)
{
  User user;
  int size;

  size = sizeof(*user) + strlen(uid) + 1 + strlen(name) + 1;
  user = malloc(size);
  memset(user, 0, size);

  user->uid = (char *)(user + 1);
  strcpy(user->uid, uid);
  user->name = user->uid + strlen(user->uid) + 1;
  strcpy(user->name, name);

  user->saucer = saucer;

  return user;
}

User User_Destroy(User user)
{
  if (user) free(user);
  return NULL;
}

char *User_GetUID(User user) { return user->uid; }
char *User_GetName(User user) { return user->name; }
Saucer User_GetSaucer(User user) { return user->saucer; }
Saucer User_SetSaucer(User user, Saucer saucer) { return user->saucer = saucer; }

Genre Genre_Create(char *name)
{
  Genre genre;
  int size, w, h;

  size = sizeof(*genre) + strlen(name) + 1;
  genre = malloc(size);
  memset(genre, 0, size);

  genre->name = (char *)(genre + 1);
  strcpy(genre->name, name);

  genre->image = font_get_image_string(name, &w, &h, 0xffffff40, 0xffffffff);
  genre->image_width  = w;
  genre->image_height = h;

  genre->questions = ObjList_Create();

  return genre;
}

Genre Genre_Destroy(Genre genre)
{
  if (genre) {
    if (genre->questions) ObjList_Destroy(genre->questions);
    free(genre);
  }
  return NULL;
}

char *Genre_GetName(Genre genre) { return genre->name; }
int Genre_GetImageWidth(Genre genre) { return genre->image_width; }
int Genre_GetImageHeight(Genre genre) { return genre->image_height; }
unsigned char *Genre_GetImage(Genre genre) { return genre->image; }
ObjList Genre_GetQuestions(Genre genre) { return genre->questions; }

Question Question_Create(char *qid, int score, Genre genre)
{
  Question question;
  int size, w, h;
  char s[32];

  size = sizeof(*question) + strlen(qid) + 1;
  question = malloc(size);
  memset(question, 0, size);

  question->qid = (char *)(question + 1);
  strcpy(question->qid, qid);

  question->score = score;

  sprintf(s, "%d", score);
  question->image = font_get_image_string(s, &w, &h, 0xffffff40, 0xffffffff);
  question->image_width  = w;
  question->image_height = h;

  question->genre = genre;

  return question;
}

Question Question_Destroy(Question question)
{
  if (question) free(question);
  return NULL;
}

char *Question_GetQID(Question question) { return question->qid; }
int Question_GetScore(Question question) { return question->score; }
int Question_GetImageWidth(Question question) { return question->image_width; }
int Question_GetImageHeight(Question question) { return question->image_height; }
unsigned char *Question_GetImage(Question question) { return question->image; }
Genre Question_GetGenre(Question question) { return question->genre; }

Reply Reply_Create(User user, Question question, int t)
{
  Reply reply;

  reply = malloc(sizeof(*reply));
  memset(reply, 0, sizeof(*reply));

  reply->user = user;
  reply->question = question;
  reply->t = t;

  return reply;
}

Reply Reply_Destroy(Reply reply)
{
  if (reply) free(reply);
  return NULL;
}

User Reply_GetUser(Reply reply) { return reply->user; }
Question Reply_GetQuestion(Reply reply) { return reply->question; }
int Reply_GetTime(Reply reply) { return reply->t; }

Clear Clear_Create(User user, Question question, int t)
{
  Clear clear;

  clear = malloc(sizeof(*clear));
  memset(clear, 0, sizeof(*clear));

  clear->user = user;
  clear->question = question;
  clear->t = t;

  return clear;
}

Clear Clear_Destroy(Clear clear)
{
  if (clear) free(clear);
  return NULL;
}

User Clear_GetUser(Clear clear) { return clear->user; }
Question Clear_GetQuestion(Clear clear) { return clear->question; }
int Clear_GetTime(Clear clear) { return clear->t; }

User User_ListSearch(ObjList list, char *uid)
{
  User user;
  ObjListData obj;
  for (obj = ObjList_GetStart(list);
       !ObjList_IsEndEdge(list, obj);
       obj = ObjListData_GetNext(obj)) {
    user = (User)ObjListData_GetObj(obj);
    if (!strcmp(user->uid, uid))
      return user;
  }
  return NULL;
}

Genre Genre_ListSearch(ObjList list, char *name)
{
  Genre genre;
  ObjListData obj;
  for (obj = ObjList_GetStart(list);
       !ObjList_IsEndEdge(list, obj);
       obj = ObjListData_GetNext(obj)) {
    genre = (Genre)ObjListData_GetObj(obj);
    if (!strcmp(genre->name, name))
      return genre;
  }
  return NULL;
}

Question Question_ListSearch(ObjList list, char *qid)
{
  Question question;
  ObjListData obj;
  for (obj = ObjList_GetStart(list);
       !ObjList_IsEndEdge(list, obj);
       obj = ObjListData_GetNext(obj)) {
    question = (Question)ObjListData_GetObj(obj);
    if (!strcmp(question->qid, qid))
      return question;
  }
  return NULL;
}

Question Question_ListSearchAll(ObjList list, char *qid)
{
  Genre genre;
  Question question;
  ObjListData obj;
  for (obj = ObjList_GetStart(list);
       !ObjList_IsEndEdge(list, obj);
       obj = ObjListData_GetNext(obj)) {
    genre = (Genre)ObjListData_GetObj(obj);
    question = Question_ListSearch(genre->questions, qid);
    if (question)
      return question;
  }
  return NULL;
}

Reply Reply_ListSearch(ObjList list, User user, Question question)
{
  Reply reply;
  ObjListData obj;
  for (obj = ObjList_GetStart(list);
       !ObjList_IsEndEdge(list, obj);
       obj = ObjListData_GetNext(obj)) {
    reply = (Reply)ObjListData_GetObj(obj);
    if ((reply->user == user) && (reply->question == question))
      return reply;
  }
  return NULL;
}

Clear Clear_ListSearch(ObjList list, User user, Question question)
{
  Clear clear;
  ObjListData obj;
  for (obj = ObjList_GetStart(list);
       !ObjList_IsEndEdge(list, obj);
       obj = ObjListData_GetNext(obj)) {
    clear = (Clear)ObjListData_GetObj(obj);
    if ((clear->user == user) && (clear->question == question))
      return clear;
  }
  return NULL;
}
